/*
 * Copyright 2011 Red Hat, Inc.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef CALLQUEUE_H_
#define CALLQUEUE_H_

#include "verto-jsonrpc.h"
#include "dataqueue.h"

typedef struct callqueue callqueue;

callqueue *
callqueue_new(dataqueue *dq);

/* Always steals a reference to params */
bool
callqueue_push(callqueue *cq, int id, const char *method, json_object *params,
               jsonrpc_response_cb *resp, void *misc, jsonrpc_free *mfree);

void
callqueue_remove_notifications(callqueue *cq);

void
callqueue_move(callqueue *from, callqueue *to);

const char *
callqueue_as_batch(callqueue *cq);

bool
callqueue_respond(callqueue *cq, int id, json_object *response, bool error);

void
callqueue_free(callqueue *cq);

#endif /* CALLQUEUE_H_ */
