/*
 * Copyright 2011 Red Hat, Inc.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef rpcobject_H_
#define rpcobject_H_

#include "common.h"

typedef struct rpcobject rpcobject;
typedef enum rpcobject_type {
	rpcobject_type_unknown      = 0,
	rpcobject_type_request      = 1 << 0,
	rpcobject_type_call         = 1 << 1,
	rpcobject_type_notification = 1 << 2,
	rpcobject_type_response     = 1 << 3,
	rpcobject_type_result       = 1 << 4,
	rpcobject_type_error        = 1 << 5,
} rpcobject_type;

rpcobject *
rpcobject_new(json_object *object);

rpcobject_type
rpcobject_get_type(rpcobject *obj);

bool
rpcobject_is_type(rpcobject *obj, rpcobject_type types);

json_object *
rpcobject_get_id(rpcobject *obj);

int
rpcobject_get_id_int(rpcobject *obj);

json_type
rpcobject_get_id_type(rpcobject *obj);

const char *
rpcobject_get_method(rpcobject *obj);

json_object *
rpcobject_get_params(rpcobject *obj);

json_object *
rpcobject_get_result(rpcobject *obj);

json_object *
rpcobject_get_error(rpcobject *obj);

int
rpcobject_get_error_code(rpcobject *obj);

const char *
rpcobject_get_error_message(rpcobject *obj);

json_object *
rpcobject_get_error_data(rpcobject *obj);

void
rpcobject_free(rpcobject *obj);

#endif /* rpcobject_H_ */
