/*
 * Copyright 2011 Red Hat, Inc.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef JSONRPC_H_
#define JSONRPC_H_

#include <verto.h>
#include <json-c/json.h>
#include <stdbool.h>

#define JSONRPC_ERRCODE_INVALID_REQUEST  -32600
#define JSONRPC_ERRCODE_METHOD_NOT_FOUND -32601
#define JSONRPC_ERRCODE_INVALID_PARAMS   -32602
#define JSONRPC_ERRCODE_INTERNAL_ERROR   -32603
#define JSONRPC_ERRCODE_PARSE_ERROR      -32700

typedef struct jsonrpc_batch jsonrpc_batch;

typedef bool
jsonrpc_handler_cb(verto_ctx *ctx, const char *method, json_object *params,
                   void *misc, json_object **response);

typedef void
jsonrpc_response_cb(json_object *response, bool error, void *misc);

typedef void
jsonrpc_error_cb(verto_ev *ev, int errno, void *misc);

typedef void
jsonrpc_free(void *mem);

verto_ev *
jsonrpc_init(verto_ctx *ctx, int infd, int outfd, size_t maxsize,
             jsonrpc_error_cb *errcb, void *misc, jsonrpc_free *free);

bool
jsonrpc_set_handler(verto_ev *ev, const char *method,
                    jsonrpc_handler_cb *handler,
                    void *misc, jsonrpc_free *free);

/* Creates a new batch object which can do multiple calls at once. */
jsonrpc_batch *
jsonrpc_batch_new(verto_ev *ev);

bool
jsonrpc_batch_send(jsonrpc_batch *batch);

void
jsonrpc_batch_free(jsonrpc_batch *batch);

/* Always steals a reference to params */
bool
jsonrpc_call(verto_ev *ev, jsonrpc_batch *batch, const char *method,
             json_object *params, jsonrpc_response_cb *response,
             void *misc, jsonrpc_free *free);

#endif /* JSONRPC_H_ */
